=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-03-09 07:59:36 +0000
+++ bzrlib/bzrdir.py	2006-04-04 10:20:54 +0000
@@ -1536,11 +1536,12 @@
         return BzrDir.open(self.bzrdir.root_transport.base)
 
     def _convert_to_prefixed(self):
-        from bzrlib.store import hash_prefix
+        from bzrlib.store import TransportStore
         self.bzrdir.transport.delete('branch-format')
         for store_name in ["weaves", "revision-store"]:
-            self.pb.note("adding prefixes to %s" % store_name) 
+            self.pb.note("adding prefixes to %s" % store_name)
             store_transport = self.bzrdir.transport.clone(store_name)
+            store = TransportStore(store_transport, prefixed=True)
             for filename in store_transport.list_dir('.'):
                 if (filename.endswith(".weave") or
                     filename.endswith(".gz") or
@@ -1548,7 +1549,7 @@
                     file_id = os.path.splitext(filename)[0]
                 else:
                     file_id = filename
-                prefix_dir = hash_prefix(file_id)
+                prefix_dir = store.hash_prefix(file_id)
                 # FIXME keep track of the dirs made RBC 20060121
                 try:
                     store_transport.move(filename, prefix_dir + '/' + filename)

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-03-09 00:16:20 +0000
+++ bzrlib/lockable_files.py	2006-04-04 10:20:54 +0000
@@ -16,6 +16,7 @@
 
 from cStringIO import StringIO
 import codecs
+#import traceback
 
 import bzrlib
 from bzrlib.decorators import *
@@ -23,7 +24,8 @@
 from bzrlib.errors import LockError, ReadOnlyError
 from bzrlib.osutils import file_iterator, safe_unicode
 from bzrlib.symbol_versioning import *
-from bzrlib.trace import mutter
+from bzrlib.symbol_versioning import deprecated_method, zero_eight
+from bzrlib.trace import mutter, note
 import bzrlib.transactions as transactions
 
 # XXX: The tracking here of lock counts and whether the lock is held is
@@ -80,7 +82,7 @@
         self._lock_mode = None
         self._lock_count = 0
         esc_name = self._escape(lock_name)
-        self._lock = lock_class(transport, esc_name, 
+        self._lock = lock_class(transport, esc_name,
                                 file_modebits=self._file_mode,
                                 dir_modebits=self._dir_mode)
 
@@ -95,6 +97,16 @@
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,
                            self._transport)
+    def __str__(self):
+        return 'LockableFiles(%s, %s)' % (self.lock_name, self._transport.base)
+
+    def __del__(self):
+        if self.is_locked():
+            # XXX: This should show something every time, and be suitable for
+            # headless operation and embedding
+            from warnings import warn
+            warn("file group %r was not explicitly unlocked" % self)
+            self._lock.unlock()
 
     def _escape(self, file_or_path):
         if not isinstance(file_or_path, basestring):
@@ -201,6 +213,8 @@
             self._lock_count += 1
         else:
             self._lock.lock_write()
+            #note('write locking %s', self)
+            #traceback.print_stack()
             self._lock_mode = 'w'
             self._lock_count = 1
             self._set_transaction(transactions.WriteTransaction())
@@ -213,6 +227,8 @@
             self._lock_count += 1
         else:
             self._lock.lock_read()
+            #note('read locking %s', self)
+            #traceback.print_stack()
             self._lock_mode = 'r'
             self._lock_count = 1
             self._set_transaction(transactions.ReadOnlyTransaction())
@@ -226,6 +242,8 @@
         if self._lock_count > 1:
             self._lock_count -= 1
         else:
+            #note('unlocking %s', self)
+            #traceback.print_stack()
             self._finish_transaction()
             self._lock.unlock()
             self._lock_mode = self._lock_count = None

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-03-09 21:37:04 +0000
+++ bzrlib/merge.py	2006-04-04 10:20:54 +0000
@@ -280,8 +280,13 @@
         for file_id in old_entries:
             entry = old_entries[file_id]
             path = id2path(file_id)
+            if file_id in self.base_tree.inventory:
+                executable = getattr(self.base_tree.inventory[file_id], 'executable', False)
+            else:
+                executable = getattr(entry, 'executable', False)
             new_inventory[file_id] = (path, file_id, entry.parent_id, 
-                                      entry.kind)
+                                      entry.kind, executable)
+                                      
             by_path[path] = file_id
         
         deletions = 0
@@ -304,7 +309,11 @@
                 parent = by_path[os.path.dirname(path)]
             abspath = pathjoin(self.this_tree.basedir, path)
             kind = bzrlib.osutils.file_kind(abspath)
-            new_inventory[file_id] = (path, file_id, parent, kind)
+            if file_id in self.base_tree.inventory:
+                executable = getattr(self.base_tree.inventory[file_id], 'executable', False)
+            else:
+                executable = False
+            new_inventory[file_id] = (path, file_id, parent, kind, executable)
             by_path[path] = file_id 
 
         # Get a list in insertion order

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-10 02:06:40 +0000
+++ bzrlib/repository.py	2006-04-04 10:20:54 +0000
@@ -33,7 +33,7 @@
 from bzrlib.store.versioned import VersionedFileStore, WeaveStore
 from bzrlib.store.text import TextStore
 from bzrlib.symbol_versioning import *
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, note
 from bzrlib.tree import RevisionTree
 from bzrlib.tsort import topo_sort
 from bzrlib.testament import Testament
@@ -96,7 +96,7 @@
             else:
                 # yes, this is not suitable for adding with ghosts.
                 self.add_inventory(rev_id, inv, rev.parent_ids)
-        self._revision_store.add_revision(rev, self.get_transaction())   
+        self._revision_store.add_revision(rev, self.get_transaction())
 
     @needs_read_lock
     def _all_possible_ids(self):
@@ -143,7 +143,7 @@
         getting file texts, inventories and revisions, then
         this construct will accept instances of those things.
         """
-        object.__init__(self)
+        super(Repository, self).__init__()
         self._format = _format
         # the following are part of the public API for Repository:
         self.bzrdir = a_bzrdir
@@ -156,6 +156,8 @@
         # 
         self.control_store = control_store
         self.control_weaves = control_store
+        # TODO: make sure to construct the right store classes, etc, depending
+        # on whether escaping is required.
 
     def lock_write(self):
         self.control_files.lock_write()
@@ -226,6 +228,20 @@
         self.copy_content_into(result, revision_id, basis)
         return result
 
+    # XXX: Is this actually called?
+    @needs_read_lock
+    def copy(self, destination):
+        raise NotImplementedError("Repository.copy() no longer required?")
+        destination.lock_write()
+        try:
+            copy_all(self.weave_store, destination.weave_store)
+            note('copying inventories')
+            destination.control_weaves.copy_multi(self.control_weaves,
+                ['inventory'])
+            copy_all(self.revision_store, destination.revision_store)
+        finally:
+            destination.unlock()
+
     @needs_read_lock
     def has_revision(self, revision_id):
         """True if this repository has a copy of the revision."""

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2006-03-02 06:23:15 +0000
+++ bzrlib/store/__init__.py	2006-04-04 10:20:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Development Ltd
+# Copyright (C) 2005, 2006 by Canonical Development Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -206,7 +206,10 @@
 
     def has_id(self, fileid, suffix=None):
         """See Store.has_id."""
-        return self._transport.has_any(self._id_to_names(fileid, suffix))
+        paths = self._id_to_names(fileid, suffix)
+        if not self._transport.has_any(paths):
+            return False
+        return True
 
     def _get_name(self, fileid, suffix=None):
         """A special check, which returns the name of an existing file.
@@ -238,7 +241,8 @@
         raise KeyError(fileid)
 
     def __init__(self, a_transport, prefixed=False, compressed=False,
-                 dir_mode=None, file_mode=None):
+                 dir_mode=None, file_mode=None,
+                 escaped=False):
         assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
         self._transport = a_transport
@@ -246,15 +250,25 @@
         # FIXME RBC 20051128 this belongs in TextStore.
         self._compressed = compressed
         self._suffixes = set()
+        self._escaped = escaped
 
         # It is okay for these to be None, it just means they
         # will just use the filesystem defaults
         self._dir_mode = dir_mode
         self._file_mode = file_mode
 
+    def _unescape(self, file_id):
+        """If filename escaping is enabled for this store, unescape and return the filename."""
+        if self._escaped:
+            return urllib.unquote(file_id)
+        else:
+            return file_id
+
     def _iter_files_recursive(self):
         """Iterate through the files in the transport."""
         for quoted_relpath in self._transport.iter_files_recursive():
+            # transport iterator always returns quoted paths, regardless of
+            # escaping
             yield urllib.unquote(quoted_relpath)
 
     def __iter__(self):
@@ -269,7 +283,7 @@
                     if name.endswith('.' + suffix):
                         skip = True
             if not skip:
-                yield name
+                yield self._unescape(name)
 
     def __len__(self):
         return len(list(self.__iter__()))
@@ -284,11 +298,37 @@
         else:
             suffixes = []
         if self._prefixed:
-            path = [hash_prefix(fileid) + fileid]
+            # hash_prefix adds the '/' separator
+            prefix = self.hash_prefix(fileid)
         else:
-            path = [fileid]
-        path.extend(suffixes)
-        return transport.urlescape(u'.'.join(path))
+            prefix = ''
+        if self._escaped:
+            fileid = self._escape_file_id(fileid)
+        path = prefix + fileid
+        full_path = u'.'.join([path] + suffixes)
+        return transport.urlescape(full_path)
+
+    def _escape_file_id(self, file_id):
+        """Turn a file id into a filesystem safe string.
+
+        This is similar to a plain urllib.quote, except
+        it uses specific safe characters, so that it doesn't
+        have to translate a lot of valid file ids.
+        """
+        if not self._escaped:
+            return file_id
+        # @ does not get escaped. This is because it is a valid
+        # filesystem character we use all the time, and it looks
+        # a lot better than seeing %40 all the time.
+        if isinstance(file_id, unicode):
+            file_id = file_id.encode('utf-8')
+        return urllib.quote(file_id, safe='@')
+
+    def hash_prefix(self, fileid):
+        # fileid should be unescaped
+        if self._escaped:
+            fileid = self._escape_file_id(fileid)
+        return "%02x/" % (adler32(fileid) & 0xff)
 
     def __repr__(self):
         if self._transport is None:
@@ -331,9 +371,3 @@
 def copy_all(store_from, store_to, pb=None):
     """Copy all ids from one store to another."""
     store_to.copy_all_ids(store_from, pb)
-
-
-def hash_prefix(fileid):
-    return "%02x/" % (adler32(fileid) & 0xff)
-
-

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2006-02-13 01:28:03 +0000
+++ bzrlib/store/text.py	2006-04-04 10:20:54 +0000
@@ -14,8 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""
-A store that keeps the full text of every version.
+"""A store that keeps the full text of every version.
 
 This store keeps uncompressed versions of the full text. It does not
 do any sort of delta compression.
@@ -23,7 +22,6 @@
 
 import os
 import bzrlib.store
-from bzrlib.store import hash_prefix
 from bzrlib.trace import mutter
 from bzrlib.errors import BzrError, NoSuchFile, FileExists
 
@@ -95,13 +93,13 @@
             raise KeyError(fileid + '-' + str(suffix))
 
         try:
-            result = other._transport.copy_to([path], self._transport, 
+            result = other._transport.copy_to([path], self._transport,
                                               mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
+                self._transport.mkdir(self.hash_prefix(fileid)[:-1], mode=self._dir_mode)
             except FileExists:
                 pass
             result = other._transport.copy_to([path], self._transport,

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2006-03-09 04:53:03 +0000
+++ bzrlib/store/versioned/__init__.py	2006-04-04 10:20:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,12 +24,13 @@
 import urllib
 
 from bzrlib.weavefile import read_weave, write_weave_v5
-from bzrlib.weave import WeaveFile
-from bzrlib.store import TransportStore, hash_prefix
+from bzrlib.weave import WeaveFile, Weave
+from bzrlib.store import TransportStore
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.errors import NoSuchFile, FileExists
 from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter
+import bzrlib.ui
 
 
 class VersionedFileStore(TransportStore):
@@ -37,10 +38,11 @@
 
     def __init__(self, transport, prefixed=False, precious=False,
                  dir_mode=None, file_mode=None,
-                 versionedfile_class=WeaveFile):
-        super(WeaveStore, self).__init__(transport,
+                 versionedfile_class=WeaveFile,
+                 escaped=False):
+        super(VersionedFileStore, self).__init__(transport,
                 dir_mode=dir_mode, file_mode=file_mode,
-                prefixed=prefixed, compressed=False)
+                prefixed=prefixed, compressed=False, escaped=escaped)
         self._precious = precious
         self._versionedfile_class = versionedfile_class
 
@@ -61,10 +63,7 @@
 
     def filename(self, file_id):
         """Return the path relative to the transport root."""
-        if self._prefixed:
-            return hash_prefix(file_id) + urllib.quote(file_id)
-        else:
-            return urllib.quote(file_id)
+        return self._relpath(file_id)
 
     def __iter__(self):
         suffixes = self._versionedfile_class.get_suffixes()
@@ -72,10 +71,13 @@
         for relpath in self._iter_files_recursive():
             for suffix in suffixes:
                 if relpath.endswith(suffix):
-                    id = os.path.basename(relpath[:-len(suffix)])
-                    if not id in ids:
-                        yield id
-                        ids.add(id)
+                    # TODO: use standard remove_suffix function
+                    escaped_id = os.path.basename(relpath[:-len(suffix)])
+                    file_id = self._unescape(escaped_id)
+                    if file_id not in ids:
+                        ids.add(file_id)
+                        yield file_id
+                    break # only one suffix can match
 
     def has_id(self, fileid):
         suffixes = self._versionedfile_class.get_suffixes()
@@ -99,6 +101,19 @@
             self._transport.delete(filename + suffix)
         self._clear_cache_id(file_id, transaction)
 
+    def _get(self, file_id):
+        return self._transport.get(self.filename(file_id))
+
+    def _put(self, file_id, f):
+        fn = self.filename(file_id)
+        try:
+            return self._transport.put(fn, f, mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
+            return self._transport.put(fn, f, mode=self._file_mode)
+
     def get_weave(self, file_id, transaction):
         weave = transaction.map.find_weave(file_id)
         if weave is not None:
@@ -124,7 +139,6 @@
 
         Returned as a list of lines.
         """
-        assert 0
         w = self.get_weave(file_id, transaction)
         return w.get_lines(rev_id)
     
@@ -146,7 +160,7 @@
                 # unexpected error - NoSuchFile is raised on a missing dir only and that
                 # only occurs when we are prefixed.
                 raise
-            self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
+            self._transport.mkdir(self.hash_prefix(file_id), mode=self._dir_mode)
             weave = self._versionedfile_class(self.filename(file_id), self._transport, self._file_mode, create=True)
         return weave
 
@@ -246,17 +260,16 @@
             # we are copying single objects, and there may be open tranasactions
             # so again with the passthrough
             to_transaction = PassThroughTransaction()
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
         for count, f in enumerate(file_ids):
             mutter("copy weave {%s} into %s", f, self)
-            if pb:
-                pb.update('copy', count, len(file_ids))
+            pb.update('copy', count, len(file_ids))
             # if we have it in cache, its faster.
             # joining is fast with knits, and bearable for weaves -
             # indeed the new case can be optimised if needed.
             target = self._make_new_versionedfile(f, to_transaction)
             target.join(from_store.get_weave(f, from_transaction))
-        if pb:
-            pb.clear()
+        pb.finished()
 
     def total_size(self):
         count, bytes =  super(VersionedFileStore, self).total_size()

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-03-09 21:04:59 +0000
+++ bzrlib/tests/__init__.py	2006-04-04 10:20:54 +0000
@@ -911,6 +911,7 @@
                    'bzrlib.tests.test_diff',
                    'bzrlib.tests.test_doc_generate',
                    'bzrlib.tests.test_errors',
+                   'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_gpg',
                    'bzrlib.tests.test_graph',

=== added file 'bzrlib/tests/test_escaped_store.py'
--- bzrlib/tests/test_escaped_store.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_escaped_store.py	2006-04-04 10:20:54 +0000
@@ -0,0 +1,106 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test Escaped Stores."""
+
+from cStringIO import StringIO
+import os
+import gzip
+
+from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
+from bzrlib.store import copy_all
+from bzrlib.store.text import TextStore
+from bzrlib.tests import TestCaseWithTransport
+import bzrlib.transport
+
+
+class TestEscaped(TestCaseWithTransport):
+    """Mixin template class that provides some common tests for stores"""
+
+    def get_store(self, prefixed=False, escaped=True):
+        t = bzrlib.transport.get_transport(self.get_url())
+        return TextStore(t, prefixed=prefixed, escaped=escaped)
+
+    def test_paths(self):
+        text_store = self.get_store()
+
+        self.assertEqual('a', text_store._relpath('a'))
+        self.assertEqual('a', text_store._relpath(u'a'))
+        self.assertEqual('%2520', text_store._relpath(' '))
+        self.assertEqual('%40%253A%253C%253E', text_store._relpath('@:<>'))
+        self.assertEqual('%25C3%25A5', text_store._relpath(u'\xe5'))
+
+    def test_prefixed(self):
+        # Prefix should be determined by unescaped string
+        text_store = self.get_store(prefixed=True)
+
+        # hash_prefix() is not defined for unicode characters
+        # it is only defined for byte streams.
+        # so hash_prefix() needs to operate on *at most* utf-8
+        # encoded. However urlescape() does both encoding to utf-8
+        # and urllib quoting, so we will use the escaped form
+        # as the path passed to hash_prefix
+
+        self.assertEqual('62/a', text_store._relpath('a'))
+        self.assertEqual('88/%2520', text_store._relpath(' '))
+        self.assertEqual('12/%40%253A%253C%253E',
+                text_store._relpath('@:<>'))
+        self.assertEqual('37/%25C3%25A5', text_store._relpath(u'\xe5'))
+
+    def test_files(self):
+        text_store = self.get_store(prefixed=True)
+
+        text_store.add(StringIO('a'), 'a')
+        self.failUnlessExists('62/a')
+
+        text_store.add(StringIO('space'), ' ')
+        self.failUnlessExists('88/%20')
+        self.assertEquals('space', text_store.get(' ').read())
+
+        text_store.add(StringIO('surprise'), '@:<>')
+        self.failUnlessExists('12/@%3A%3C%3E')
+        self.assertEquals('surprise', text_store.get('@:<>').read())
+
+        text_store.add(StringIO('unicode'), u'\xe5')
+        self.failUnlessExists('37/%C3%A5')
+        self.assertEquals('unicode', text_store.get(u'\xe5').read())
+
+    def test_weave(self):
+        from bzrlib.store.versioned import WeaveStore
+        from bzrlib.transactions import PassThroughTransaction
+
+        trans = PassThroughTransaction()
+
+        t = bzrlib.transport.get_transport(self.get_url())
+        weave_store = WeaveStore(t, prefixed=True, escaped=True)
+
+        weave_store.add_text('a', 'r', ['a'], [], trans)
+        self.failUnlessExists('62/a.weave')
+        self.assertEqual(['a'], weave_store.get_lines('a', 'r', trans))
+
+        weave_store.add_text(' ', 'r', ['space'], [], trans)
+        self.failIfExists('21/ .weave')
+        self.failUnlessExists('88/%20.weave')
+        self.assertEquals(['space'], weave_store.get_lines(' ', 'r', trans))
+
+        weave_store.add_text('@:<>', 'r', ['surprise'], [], trans)
+        self.failUnlessExists('12/@%3A%3C%3E.weave')
+        self.assertEquals(['surprise'], weave_store.get_lines('@:<>', 'r', trans))
+
+        weave_store.add_text(u'\xe5', 'r', ['unicode'], [], trans)
+        self.failUnlessExists('37/%C3%A5.weave')
+        self.assertEquals(['unicode'], weave_store.get_lines(u'\xe5', 'r', trans))
+

=== removed file 'bzrlib/tests/test_reweave.py.moved'
--- bzrlib/tests/test_reweave.py.moved	2006-03-06 06:06:07 +0000
+++ bzrlib/tests/test_reweave.py.moved	1970-01-01 00:00:00 +0000
@@ -1,36 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""Test reweave code.
-
-Reweave takes two weaves containing a partial view of history and combines
-them into a single weave containing all the information.  This can include 
-
- - versions recorded in only one file
-
- - versions with different (but not contradictory) lists of parent 
-   revisions
-
-It is an error if either of these conditions occur:
-
- - contradictory ancestry graphs, e.g.
-   - v1 is an ancestor of v2 in one weave, and vice versa in the other
-   - different text for any version 
-"""
-
-import os
-import sys
-

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-03-09 07:59:36 +0000
+++ bzrlib/transport/__init__.py	2006-04-04 10:20:54 +0000
@@ -24,6 +24,7 @@
 from copy import deepcopy
 from stat import *
 import sys
+import urllib
 from unittest import TestSuite
 
 from bzrlib.trace import mutter
@@ -597,8 +598,8 @@
 
 def urlescape(relpath):
     """Escape relpath to be a valid url."""
-    # TODO utf8 it first. utf8relpath = relpath.encode('utf8')
-    import urllib
+    if isinstance(relpath, unicode):
+        relpath = relpath.encode('utf-8')
     return urllib.quote(relpath)
 
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-03-09 07:14:10 +0000
+++ bzrlib/workingtree.py	2006-04-04 10:20:54 +0000
@@ -88,8 +88,8 @@
 from bzrlib.symbol_versioning import *
 from bzrlib.textui import show_status
 import bzrlib.tree
-from bzrlib.trace import mutter
 from bzrlib.transform import build_tree
+from bzrlib.trace import mutter, note
 from bzrlib.transport import get_transport
 from bzrlib.transport.local import LocalTransport
 import bzrlib.ui
@@ -1136,7 +1136,7 @@
                                       InventoryFile,
                                       InventoryLink)
         inv = Inventory(self.get_root_id())
-        for path, file_id, parent, kind in new_inventory_list:
+        for path, file_id, parent, kind, executable in new_inventory_list:
             name = os.path.basename(path)
             if name == "":
                 continue
@@ -1144,7 +1144,9 @@
             if kind == 'directory':
                 inv.add(InventoryDirectory(file_id, name, parent))
             elif kind == 'file':
-                inv.add(InventoryFile(file_id, name, parent))
+                ie = InventoryFile(file_id, name, parent)
+                ie.executable = executable
+                inv.add(ie)
             elif kind == 'symlink':
                 inv.add(InventoryLink(file_id, name, parent))
             else:

